# Copyright (C) 2018, 2019 Nathan Matthew Miller

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import tkinter as tk

# Take Tk's index value, which is formatted as a string, and return a
# tuple with the line and character positions.
#
def index_to_int( index ):

    (line, pos) = index.split( '.' )
    return ( int(line), int(pos) )


class Model:

    def __init__( self, view ):

        self.view = view
        self.doc  = []
        
        return


    def create_model( self, content ):

        self.doc.clear()

        for entry in content:
            entity_id   = entry[0]
            entity_name = entry[1]
            e_ordering  = entry[2]
            
            self.doc.append( (entity_id, entity_name, e_ordering) )
        return


    def remove_line( self, line_no ):

        del self.doc[line_no-1]
        return


    # Load the model into the Text widget.
    #
    def load_model( self, entity_name ):

        for rel in self.doc:
            self.view.text.add_relation( rel[0], rel[1] )

        self.view.set_title( entity_name )
        return
    

    # Load a KB entity into the Text widget.
    # Assumption is that the Text widget is empty.
    #
    def load_entity( self, e_name, entity_details, relations ):

        self.create_model( entity_details[2] )
        self.load_model( e_name )

        for r in relations:
            self.view.rel_view.add_relation( r['child'], r['name'] )
        return


    # Return the textual content of the model.
    #
    def get_content( self ):

        content = ""
        return content


    # Return the title of this entity.
    #
    def get_title( self ):

        return self.view.entity_name.get()
    

    # A line in the model has been updated, update the view as well.
    #
    def update_view( self, line_no ):

        # Remove the text of the line (except for the newline) and insert
        # the current text from the model (except for the newline).
        self.view.text.delete( f"{line_no}.0", f"{line_no}.lineend" )
        line_seg = self.doc[ line_no-1 ]
        self.view.text.insert( f"{line_no}.0", line_seg[1].rstrip() )
        return
    

    # Update the view by adding a link to another entity.
    # The model is just an intermediary here...
    #
    def add_relationship_link( self, link_id, entity ):

        self.view.rel_view.add_relation( link_id, entity[1] )
        return


    # Update the model and then view with the newly added category.
    #
    def category_add( self, link_id, entity ):

        self.view.text.add_relation( link_id, entity[1] )
        return
    
