# Copyright (C) 2018, 2019 Nathan Matthew Miller

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import tkinter as tk
from tkinter.font import *

# Take Tk's index value, which is formatted as a string, and return a
# tuple with the line and character positions.
#
def index_to_int( index ):

    (line, pos) = index.split( '.' )
    return ( int(line), int(pos) )


class ViewBox( tk.Frame ):

    def __init__( self, top, ctrl, text_height=4 ):

        super().__init__( top )

        self.ctrl = ctrl
        
        ryscroll = tk.Scrollbar( self )
        ryscroll.pack( side=tk.RIGHT, fill=tk.Y )
        self.rel = tk.Text( self, yscrollcommand=ryscroll.set, height=text_height )
        ryscroll.config( command=self.rel.yview )
        self.rel.pack( expand=True, fill=tk.Y )

        # Prevent inadvertent editting of the widget.
        self.rel.bind( "<KeyPress>", lambda event: "break" )

        self.xfont = Font( family='Helvetica', size=-12, weight='bold' )

        self.rels = [] # List of all displayed relationships
        return
    

    def mouse_enter( self, event ):
        self.rel.config( cursor="hand2" )
        return
    

    def mouse_leave( self, event ):
        self.rel.config( cursor="xterm" )
        return


    def rel_delete( self, event ):
        self.rel.config( cursor="xterm" ) # reset the cursor
        
        i = event.widget.index( f"@{event.x},{event.y}" )
        (line,pos) = index_to_int( i )

        id = self.rels[ line-1 ][0]
        del self.rels[ line-1 ]

        self.rel.delete( f"{line}.0", f"{line+1}.0" )
        self.ctrl.remove_relation( id )
        return


    def rel_follow( self, event ):
        i = event.widget.index( f"@{event.x},{event.y}" )
        (line,pos) = index_to_int( i )

        id = self.rels[ line-1 ]
        self.ctrl.show_entity( id[0] )
        return


    def show_relation( self, line, id, title ):

        self.rel.insert( f"{line}.0", "X " + title + "\n" )
        self.rel.tag_add( "HOVER", f"{line}.0", f"{line}.1" )
        
        self.rel.tag_config( "HOVER", font=self.xfont, background="red" )
        self.rel.tag_bind( "HOVER", "<Enter>", self.mouse_enter )
        self.rel.tag_bind( "HOVER", "<Leave>", self.mouse_leave )
        self.rel.tag_bind( "HOVER", "<Button-1>", self.rel_delete )

        self.rel.tag_add( "LINK", f"{line}.2", f"{line}.end" )
        self.rel.tag_config( "LINK", foreground="blue", underline=1 )
        self.rel.tag_bind( "LINK", "<Enter>", self.mouse_enter )
        self.rel.tag_bind( "LINK", "<Leave>", self.mouse_leave )
        self.rel.tag_bind( "LINK", "<Button-1>", self.rel_follow )
        return


    # Add a new relationship to the view, which is indexed by the id value.
    #
    def add_relation( self, id, title ):

        self.rels.append( (id, title) )
        line = len( self.rels )
        
        self.show_relation( line, id, title )
        return
    
