#!/usr/bin/env python3

# TKinter interface to the knowledge base.
#
# The database file to open is an optional command line argument.

# Copyright (C) 2018, 2019 Nathan Matthew Miller

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import sys

import tkinter as tk
from tkinter.font import *
from tkinter import messagebox

from TkGUI import *
import KB.Core as core


def exit_gui():
    global root
    root.quit()
    

try:
    kb_db_file = None
    if len(sys.argv) > 1:
        kb_db_file = sys.argv[1]
    
    kb = core.Core( db_file=kb_db_file )
except Exception as error:

    messagebox.showerror( "Error!", str(error) )
    sys.exit( 1 )

root = tk.Tk()

# Set the default font for widgets which display text.
#
try:
    dfont = Font( family='Bitstream Vera Sans Mono', size=10 )
except:
    dfont = Font.nametofont( "TkFixedFont" )
    dfont.configure( size=10 )

root.option_add( "*Entry.Font",   dfont )
root.option_add( "*Text.Font",    dfont )
root.option_add( "*Listbox.Font", dfont )

root.wm_title( "Exocortex Knowledge Base" )

root.wm_protocol( 'WM_DELETE_WINDOW', exit_gui )

cntl = SearchController.Controller( root, kb )
view = SearchView.View( root, cntl )
cntl.set_view( view )

tk.mainloop()

print( "Exiting" )
kb.close()
sys.exit( 0 )
